'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

require('./touch-slider.less');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = _react2.default.createClass({
    displayName: 'react-touch-slider',

    getDefaultProps: function getDefaultProps() {
        return {
            actionDistance: 30 };
    },
    getInitialState: function getInitialState() {
        return {
            currentIndex: 0,
            transition: 1, // 是否需要缓动
            action: 0,
            x0: 0 };
    },
    valiIndex: function valiIndex(index) {
        var imgs = this.props.imgs;

        // 处理循环

        var firstIndex = 0;
        var lastIndex = imgs.length - 1;
        if (index < firstIndex) index = lastIndex;else if (index > lastIndex) index = firstIndex;

        return index;
    },
    touchStart: function touchStart(e) {
        this.stopAutoPlay();
        if (this.state.x0) this.transitionEnd(); // reset

        // 设置起始触点用于跟踪拖拽
        if (e.touches.length == 1) {
            this._initialTouch = {
                clientX: e.touches[0].clientX
            };

            this.setState({
                transition: 0
            });
        }
    },
    touchMove: function touchMove(e) {
        var x0 = e.touches[0].clientX - this._initialTouch.clientX;

        if (Math.abs(x0) > 5) {
            // 响应滚动
            e.preventDefault();
            this.setState({ x0: x0 });
        }
    },
    touchEnd: function touchEnd(e) {
        var action = 0;
        var actionDistance = 30; // 触发切换动作需要拖拽的距离
        var x0 = this.state.x0;

        // 判断拖拽动作

        if (Math.abs(x0) > this.props.actionDistance) action = x0 > 0 ? -1 : 1;

        this.step(action);
    },


    // 切换动效
    step: function step(action) {
        if (this.state.action) this.transitionEnd(); // reset

        var nextState = {
            transition: 1
        };
        if (action) nextState.action = action;else nextState.x0 = 0;

        this.setState(nextState);
    },

    // 切换动效完成，更新状态
    // 并不能总是触发，需要在引用状态前(自动、手动执行下一个轮播)做检查
    transitionEnd: function transitionEnd() {
        var _state = this.state;
        var currentIndex = _state.currentIndex;
        var action = _state.action;


        this.setState({
            transition: 0,
            x0: 0,
            action: 0,
            currentIndex: this.valiIndex(currentIndex + action)
        });
        this.startAutoPlay();
    },
    componentDidMount: function componentDidMount() {
        if (this.props.autoPlayInterval) this.startAutoPlay();
    },
    startAutoPlay: function startAutoPlay() {
        var autoPlayInterval = this.props.autoPlayInterval;
        var _autoPlayIntervalId = this._autoPlayIntervalId;

        if (!autoPlayInterval || _autoPlayIntervalId) return;

        // start auto play
        this._autoPlayIntervalId = setInterval(function () {
            this.step(1);
        }.bind(this), autoPlayInterval);
    },
    stopAutoPlay: function stopAutoPlay() {
        var autoPlayInterval = this.props.autoPlayInterval;

        if (!autoPlayInterval) return;

        clearInterval(this._autoPlayIntervalId);
        this._autoPlayIntervalId = 0;
    },
    render: function render() {
        var imgs = this.props.imgs;
        var _state2 = this.state;
        var currentIndex = _state2.currentIndex;
        var action = _state2.action;
        var x0 = _state2.x0;
        var transition = _state2.transition;
        var valiIndex = this.valiIndex;
        var touchStart = this.touchStart;
        var touchMove = this.touchMove;
        var touchEnd = this.touchEnd;
        var transitionEnd = this.transitionEnd;

        // 一次性加载全部图片，防止切换时重新加载，待优化

        var items = imgs.slice(-1).concat(imgs).concat(imgs[0]).map(function (img, i) {
            var d = i - currentIndex;
            var visible = d >= 0 && d <= 2;

            return _react2.default.createElement(
                'div',
                { key: i, className: visible ? 'tslider-item' : 'tslider-none' },
                _react2.default.createElement('img', { src: img })
            );
        });

        var dots = imgs.map(function (img, i) {
            return _react2.default.createElement('i', { key: i, className: currentIndex == i ? 'on' : '' });
        });

        // transform3d 在 safari 滚动时会遮挡 position absolute
        var transitionX = action ? -action * 33.333 + '%' : x0 + 'px';

        return _react2.default.createElement(
            'div',
            { className: 'tslider',
                onTouchStart: touchStart,
                onTouchMove: touchMove,
                onTouchEnd: touchEnd },
            _react2.default.createElement(
                'div',
                { style: { WebkitTransform: 'translateX(' + transitionX + ')' },
                    onTransitionEnd: transitionEnd,
                    className: 'tslider-items ' + (transition ? 'transition' : '') },
                items
            ),
            _react2.default.createElement(
                'div',
                { className: 'tslider-dots' },
                dots
            )
        );
    }
});